import time
from netmiko.cisco_base_connection import CiscoBaseConnection


class SixwindOSBase(CiscoBaseConnection):
    def session_preparation(self):
        """Prepare the session after the connection has been established."""
        self.ansi_escape_codes = True
        self._test_channel_read()
        self.set_base_prompt()
        # Clear the read buffer
        time.sleep(0.3 * self.global_delay_factor)
        self.clear_buffer()

    def disable_paging(self, *args, **kwargs):
        """6WIND requires no-pager at the end of command, not implemented at this time."""
        pass

    def set_base_prompt(
        self, pri_prompt_terminator=">", alt_prompt_terminator="#", delay_factor=1
    ):
        """Sets self.base_prompt: used as delimiter for stripping of trailing prompt in output."""

        prompt = super().set_base_prompt(
            pri_prompt_terminator=pri_prompt_terminator,
            alt_prompt_terminator=alt_prompt_terminator,
            delay_factor=delay_factor,
        )
        prompt = prompt.strip()
        self.base_prompt = prompt
        return self.base_prompt

    def config_mode(self, config_command="edit running", pattern=""):
        """Enter configuration mode."""

        return super().config_mode(config_command=config_command, pattern=pattern)

    def commit(self, comment="", delay_factor=1):
        """
        Commit the candidate configuration.

        Raise an error and return the failure if the commit fails.
        """

        delay_factor = self.select_delay_factor(delay_factor)
        error_marker = "Failed to generate committed config"
        command_string = "commit"

        output = self.config_mode()
        output += self.send_command(
            command_string,
            strip_prompt=False,
            strip_command=False,
            delay_factor=delay_factor,
            expect_string=r"#",
        )
        output += self.exit_config_mode()

        if error_marker in output:
            raise ValueError(f"Commit failed with following errors:\n\n{output}")

        return output

    def exit_config_mode(self, exit_config="exit", pattern=r">"):
        """Exit configuration mode."""

        return super().exit_config_mode(exit_config=exit_config, pattern=pattern)

    def check_config_mode(self, check_string="#", pattern=""):
        """Checks whether in configuration mode. Returns a boolean."""

        return super().check_config_mode(check_string=check_string, pattern=pattern)

    def save_config(
        self, cmd="copy running startup", confirm=True, confirm_response="y"
    ):
        """Save Config for 6WIND"""

        return super().save_config(
            cmd=cmd, confirm=confirm, confirm_response=confirm_response
        )

    def check_enable_mode(self, *args, **kwargs):
        """6WIND has no enable mode."""

        pass

    def enable(self, *args, **kwargs):
        """6WIND has no enable mode."""

        pass

    def exit_enable_mode(self, *args, **kwargs):
        """6WIND has no enable mode."""

        pass


class SixwindOSSSH(SixwindOSBase):

    pass



from netmiko.base_connection import BaseConnection
import time


class YamahaBase(BaseConnection):
    def session_preparation(self):
        """Prepare the session after the connection has been established."""
        self._test_channel_read(pattern=r"[>#]")
        self.set_base_prompt()
        self.disable_paging(command="console lines infinity")
        time.sleep(0.3 * self.global_delay_factor)
        self.clear_buffer()

    def check_enable_mode(self, check_string="#"):
        return super().check_enable_mode(check_string=check_string)

    def enable(self, cmd="administrator", pattern=r"Password", **kwargs):
        return super().enable(cmd=cmd, pattern=pattern, **kwargs)

    def exit_enable_mode(self, exit_command="exit"):
        """
        When any changes have been made, the prompt 'Save new configuration ? (Y/N)'
        appears before exiting. Ignore this by entering 'N'.
        """
        output = ""
        if self.check_enable_mode():
            self.write_channel(self.normalize_cmd(exit_command))
            time.sleep(1)
            output = self.read_channel()
            if "(Y/N)" in output:
                self.write_channel("N")
            output += self.read_until_prompt()
            if self.check_enable_mode():
                raise ValueError("Failed to exit enable mode.")
        return output

    def check_config_mode(self, check_string="#", pattern=""):
        """Checks if the device is in administrator mode or not."""
        return super().check_config_mode(check_string=check_string, pattern=pattern)

    def config_mode(self, config_command="administrator", pattern="ssword"):
        """Enter into administrator mode and configure device."""
        return self.enable()

    def exit_config_mode(self, exit_config="exit", pattern=">"):
        """
        No action taken. Call 'exit_enable_mode()' to explicitly exit Administration
        Level.
        """
        return ""

    def save_config(self, cmd="save", confirm=False, confirm_response=""):
        """Saves Config."""
        if confirm is True:
            raise ValueError("Yamaha does not support save_config confirmation.")
        self.enable()
        # Some devices are slow so match on trailing-prompt if you can
        return self.send_command(command_string=cmd)


class YamahaSSH(YamahaBase):
    """Yamaha SSH driver."""

    pass


class YamahaTelnet(YamahaBase):
    """Yamaha Telnet driver."""

    pass



#!/usr/bin/env python

"""
This will run an ssh command successfully on an Ericsson IPPOS.
SSH must be enabled on the device

"""

from netmiko.ssh_dispatcher import ConnectHandler


def main():
    """
    This will run an ssh command successfully on an Ericsson IPPOS.
    SSH must be enabled on the device
    """

    ericsson_connect = {
        "device_type": "ericsson_ipos",
        "ip": "1.1.1.1",
        "username": "admin",
        "password": "admin",
    }

    net_connect = ConnectHandler(**ericsson_connect)
    output = net_connect.send_command("show ip int brief")
    print(output)

    output_commit = net_connect.commit()
    print(output_commit)


if __name__ == "__main__":
    main()



"""Subclass specific to Cisco FTD."""
from netmiko.cisco_base_connection import CiscoSSHConnection


class CiscoFtdSSH(CiscoSSHConnection):
    """Subclass specific to Cisco FTD."""

    def session_preparation(self):
        """Prepare the session after the connection has been established."""
        self._test_channel_read()
        self.set_base_prompt()

    def send_config_set(self, *args, **kwargs):
        """Canot change config on FTD via ssh"""
        raise NotImplementedError

    def enable(self, *args, **kwargs):
        """No enable mode on firepower ssh"""
        return ""

    def config_mode(self, *args, **kwargs):
        """No config mode on firepower ssh"""
        return ""

    def check_config_mode(self, *args, **kwargs):
        """No config mode on firepower ssh"""
        return False



import re

from netmiko.base_connection import BaseConnection


class EricssonIposSSH(BaseConnection):
    def check_enable_mode(self, check_string="#"):
        """
        Check if in enable mode. Return boolean.
        """
        return super().check_enable_mode(check_string=check_string)

    def enable(self, cmd="enable 15", pattern="ssword", re_flags=re.IGNORECASE):
        """Enter enable mode."""
        return super().enable(cmd=cmd, pattern=pattern, re_flags=re_flags)

    def disable_paging(self, command="terminal length 0", delay_factor=1):
        """Disable paging default to a Cisco CLI method.

        :param command: Device command to disable pagination of output
        :type command: str

        :param delay_factor: See __init__: global_delay_factor
        :type delay_factor: int
        """
        return super().disable_paging(command=command, delay_factor=delay_factor)

    def set_terminal_width(self, command="terminal width 512", delay_factor=1):
        """CLI terminals try to automatically adjust the line based on the width of the terminal.
        This causes the output to get distorted when accessed programmatically.

        Set terminal width to 511 which works on a broad set of devices.

        :param command: Command string to send to the device
        :type command: str

        :param delay_factor: See __init__: global_delay_factor
        :type delay_factor: int
        """
        return super().set_terminal_width(command=command, delay_factor=delay_factor)

    def send_config_set(self, config_commands=None, exit_config_mode=False, **kwargs):
        """Ericsson IPOS requires you not exit from configuration mode."""
        return super().send_config_set(
            config_commands=config_commands, exit_config_mode=exit_config_mode, **kwargs
        )

    def exit_enable_mode(self, exit_command="disable"):
        """
        Exits enable (privileged exec) mode.
        """
        return super().exit_enable_mode(exit_command=exit_command)

    def check_config_mode(self, check_string=")#", pattern=""):
        """
        Checks if the device is in configuration mode or not.
        """
        return super().check_config_mode(check_string=check_string, pattern=pattern)

    def config_mode(self, config_command="configure", pattern=""):
        """
        Enter into configuration mode on remote device.
        """
        if not pattern:
            pattern = re.escape(self.base_prompt[:16])
        return super().config_mode(config_command=config_command, pattern=pattern)

    def exit_config_mode(self, exit_config="end", pattern="#"):
        """
        Exit from configuration mode.
        Ercisson output :
            end                   Commit configuration changes and return to exec mode
        """
        return super().exit_config_mode(exit_config=exit_config, pattern=pattern)

    def save_config(self, cmd="save config", confirm=True, confirm_response="yes"):
        """Saves configuration"""
        if confirm:
            output = self.send_command_timing(
                command_string=cmd, strip_prompt=False, strip_command=False
            )

            if confirm_response:
                output += self.send_command_timing(
                    confirm_response, strip_prompt=False, strip_command=False
                )
            else:
                output += self.send_command_timing(
                    self.RETURN, strip_prompt=False, strip_command=False
                )
        else:
            output = self.send_command(
                command_string=cmd, strip_prompt=False, strip_command=False
            )
        return output

    def commit(self, confirm=False, confirm_delay=None, comment="", delay_factor=1):
        """
        Commit the candidate configuration.

        Commit the entered configuration. Raise an error and return the failure
        if the commit fails.

        Automatically enters configuration mode

        """

        delay_factor = self.select_delay_factor(delay_factor)

        if confirm_delay and not confirm:
            raise ValueError(
                "Invalid arguments supplied to commit method both confirm and check"
            )

        command_string = "commit"
        commit_marker = "Transaction committed"
        if confirm:
            if confirm_delay:
                command_string = f"commit confirmed {confirm_delay}"
            else:
                command_string = "commit confirmed"
            commit_marker = "Commit confirmed ,it will be rolled back within"

        if comment:
            if '"' in comment:
                raise ValueError("Invalid comment contains double quote")
            comment = f'"{comment}"'
            command_string += f" comment {comment}"

        output = self.config_mode()

        output += self.send_command_expect(
            command_string,
            strip_prompt=False,
            strip_command=False,
            delay_factor=delay_factor,
        )

        if commit_marker not in output:
            raise ValueError(f"Commit failed with the following errors:\n\n{output}")

        self.exit_config_mode()

        return output
